'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addWorkspacesToPackageJson = exports.updateAttribute = exports.write = exports.changeDependenciesToRelativeSyntax = exports.addComponentsToRoot = undefined;

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

/**
 * Add components as dependencies to root package.json
 */
var addComponentsToRoot = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(consumer, components) {
    var importedComponents, driver, PackageJson, componentsToAdd, registryPrefix;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, (_utils || _load_utils()).filterAsync)(components, function (component) {
              var componentMap = component.getComponentMap(consumer.bitMap);
              return componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED;
            });

          case 2:
            importedComponents = _context.sent;

            if (!(!importedComponents || !importedComponents.length)) {
              _context.next = 5;
              break;
            }

            return _context.abrupt('return');

          case 5:
            _context.next = 7;
            return consumer.driver.getDriver(false);

          case 7:
            driver = _context.sent;
            PackageJson = driver.PackageJson;
            componentsToAdd = (_ramda || _load_ramda()).default.fromPairs(importedComponents.map(function (component) {
              var locationRelativeToConsumer = consumer.getPathRelativeToConsumer(component.writtenPath);
              var locationAsUnixFormat = `./${(0, (_utils || _load_utils()).pathNormalizeToLinux)(locationRelativeToConsumer)}`;
              return [component.id.toStringWithoutVersion(), locationAsUnixFormat];
            }));
            registryPrefix = getRegistryPrefix();
            _context.next = 13;
            return PackageJson.addComponentsIntoExistingPackageJson(consumer.getPath(), componentsToAdd, registryPrefix);

          case 13:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function addComponentsToRoot(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Only imported components should be saved with relative path in package.json
 * If a component is nested or imported as a package dependency, it should be saved with the version
 */


var getPackageDependency = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(consumer, dependencyId, parentId) {
    var dependencyComponentMap, parentComponentMap;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            dependencyComponentMap = consumer.bitMap.getComponent(dependencyId);
            parentComponentMap = consumer.bitMap.getComponent(parentId);
            return _context2.abrupt('return', getPackageDependencyValue(dependencyId, parentComponentMap, dependencyComponentMap));

          case 3:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function getPackageDependency(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var changeDependenciesToRelativeSyntax = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(consumer, components, dependencies) {
    var _this = this;

    var dependenciesIds, driver, PackageJson, updateComponent;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            dependenciesIds = dependencies.map(function (dependency) {
              return dependency.id.toStringWithoutVersion();
            });
            _context4.next = 3;
            return consumer.driver.getDriver(false);

          case 3:
            driver = _context4.sent;
            PackageJson = driver.PackageJson;

            updateComponent = function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(component) {
                var componentMap, packageJson, packages;
                return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        componentMap = component.getComponentMap(consumer.bitMap);
                        packageJson = void 0;
                        _context3.prev = 2;
                        _context3.next = 5;
                        return PackageJson.load(componentMap.rootDir);

                      case 5:
                        packageJson = _context3.sent;
                        _context3.next = 11;
                        break;

                      case 8:
                        _context3.prev = 8;
                        _context3.t0 = _context3['catch'](2);
                        return _context3.abrupt('return', Promise.resolve());

                      case 11:
                        packages = component.dependencies.map(function (dependency) {
                          var dependencyId = dependency.id.toStringWithoutVersion();
                          if (dependenciesIds.includes(dependencyId)) {
                            var dependencyComponent = dependencies.find(function (d) {
                              return d.id.toStringWithoutVersion() === dependencyId;
                            });
                            var dependencyComponentMap = dependencyComponent.getComponentMap(consumer.bitMap);
                            var dependencyLocation = getPackageDependencyValue(dependencyId, componentMap, dependencyComponentMap);
                            return [dependencyId, dependencyLocation];
                          }
                          return [];
                        });

                        packageJson.setDependencies(packageJson.packageDependencies, (_ramda || _load_ramda()).default.fromPairs(packages), getRegistryPrefix());
                        return _context3.abrupt('return', packageJson.write({ override: true }));

                      case 14:
                      case 'end':
                        return _context3.stop();
                    }
                  }
                }, _callee3, _this, [[2, 8]]);
              }));

              return function updateComponent(_x9) {
                return _ref4.apply(this, arguments);
              };
            }();

            return _context4.abrupt('return', Promise.all(components.map(function (component) {
              return updateComponent(component);
            })));

          case 7:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function changeDependenciesToRelativeSyntax(_x6, _x7, _x8) {
    return _ref3.apply(this, arguments);
  };
}();

var write = function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee7(consumer, component, bitDir) {
    var force = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

    var _this2 = this;

    var writeBitDependencies = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
    var excludeRegistryPrefix = arguments[5];
    var PackageJson, getBitDependencies, bitDependencies, registryPrefix, name, packageJson;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            PackageJson = consumer.driver.getDriver(false).PackageJson;

            getBitDependencies = function () {
              var _ref6 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee6() {
                var dependenciesPackages;
                return (_regenerator || _load_regenerator()).default.wrap(function _callee6$(_context6) {
                  while (1) {
                    switch (_context6.prev = _context6.next) {
                      case 0:
                        if (writeBitDependencies) {
                          _context6.next = 2;
                          break;
                        }

                        return _context6.abrupt('return', {});

                      case 2:
                        dependenciesPackages = component.dependencies.map(function () {
                          var _ref7 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee5(dep) {
                            var packageDependency;
                            return (_regenerator || _load_regenerator()).default.wrap(function _callee5$(_context5) {
                              while (1) {
                                switch (_context5.prev = _context5.next) {
                                  case 0:
                                    _context5.next = 2;
                                    return getPackageDependency(consumer, dep.id, component.id);

                                  case 2:
                                    packageDependency = _context5.sent;
                                    return _context5.abrupt('return', [dep.id.toStringWithoutVersion(), packageDependency]);

                                  case 4:
                                  case 'end':
                                    return _context5.stop();
                                }
                              }
                            }, _callee5, _this2);
                          }));

                          return function (_x15) {
                            return _ref7.apply(this, arguments);
                          };
                        }());
                        _context6.t0 = (_ramda || _load_ramda()).default;
                        _context6.next = 6;
                        return Promise.all(dependenciesPackages);

                      case 6:
                        _context6.t1 = _context6.sent;
                        return _context6.abrupt('return', _context6.t0.fromPairs.call(_context6.t0, _context6.t1));

                      case 8:
                      case 'end':
                        return _context6.stop();
                    }
                  }
                }, _callee6, _this2);
              }));

              return function getBitDependencies() {
                return _ref6.apply(this, arguments);
              };
            }();

            _context7.next = 4;
            return getBitDependencies();

          case 4:
            bitDependencies = _context7.sent;
            registryPrefix = getRegistryPrefix();
            name = excludeRegistryPrefix ? component.id.toStringWithoutVersion().replace(/\//g, '.') : `${registryPrefix}/${component.id.toStringWithoutVersion().replace(/\//g, '.')}`;
            packageJson = new PackageJson(bitDir, {
              name,
              version: component.version,
              homepage: component._getHomepage(),
              main: component.calculateMainDistFile(),
              devDependencies: component.devPackageDependencies,
              peerDependencies: component.peerPackageDependencies,
              componentRootFolder: bitDir,
              license: `SEE LICENSE IN ${!(_ramda || _load_ramda()).default.isEmpty(component.license) ? 'LICENSE' : 'UNLICENSED'}`
            });

            packageJson.setDependencies(component.packageDependencies, bitDependencies, registryPrefix);

            return _context7.abrupt('return', packageJson.write({ override: force }));

          case 10:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function write(_x12, _x13, _x14) {
    return _ref5.apply(this, arguments);
  };
}();

var updateAttribute = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee8(consumer, componentDir, attributeName, attributeValue) {
    var PackageJson, packageJson;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            PackageJson = consumer.driver.getDriver(false).PackageJson;
            _context8.prev = 1;
            _context8.next = 4;
            return PackageJson.load(componentDir);

          case 4:
            packageJson = _context8.sent;

            packageJson[attributeName] = attributeValue;
            return _context8.abrupt('return', packageJson.write({ override: true }));

          case 9:
            _context8.prev = 9;
            _context8.t0 = _context8['catch'](1);
            return _context8.abrupt('return', Promise.resolve());

          case 12:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, this, [[1, 9]]);
  }));

  return function updateAttribute(_x16, _x17, _x18, _x19) {
    return _ref8.apply(this, arguments);
  };
}();

/**
 * Adds workspace array to package.json - only if user wants to work with yarn workspaces
 *
 * formatedComponentsPath- used to resolve import path dsl. replacing all {}->*
 * for example {components/{namespace} -> components/*
 *
 */


var addWorkspacesToPackageJson = function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee9(consumer, rootDir, componentsDefaultDirectory, dependenciesDirectory, customImportPath) {
    var formatedComponentsPath, formatedRegexPath, driver, PackageJson;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            if (!(consumer.bitJson.manageWorkspaces && consumer.bitJson.packageManager === 'yarn' && consumer.bitJson.useWorkspaces)) {
              _context9.next = 9;
              break;
            }

            formatedComponentsPath = (0, (_stringFormat || _load_stringFormat()).default)(componentsDefaultDirectory, {
              name: (_constants || _load_constants()).ASTERISK,
              scope: (_constants || _load_constants()).ASTERISK,
              namespace: (_constants || _load_constants()).ASTERISK
            });
            formatedRegexPath = formatedComponentsPath.split((_constants || _load_constants()).DEFAULT_SEPARATOR).map(function (part) {
              return (_ramda || _load_ramda()).default.contains((_constants || _load_constants()).ASTERISK, part) ? (_constants || _load_constants()).ASTERISK : part;
            }).join((_constants || _load_constants()).DEFAULT_SEPARATOR);
            _context9.next = 5;
            return consumer.driver.getDriver(false);

          case 5:
            driver = _context9.sent;
            PackageJson = driver.PackageJson;
            _context9.next = 9;
            return PackageJson.addWorkspacesToPackageJson(rootDir, formatedRegexPath, dependenciesDirectory + (_constants || _load_constants()).COMPONENTES_DEPENDECIES_REGEX, customImportPath);

          case 9:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, this);
  }));

  return function addWorkspacesToPackageJson(_x20, _x21, _x22, _x23, _x24) {
    return _ref9.apply(this, arguments);
  };
}();

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _stringFormat;

function _load_stringFormat() {
  return _stringFormat = _interopRequireDefault(require('string-format'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../component'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _componentMap;

function _load_componentMap() {
  return _componentMap = _interopRequireDefault(require('../bit-map/component-map'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = require('../../api/consumer/lib/global-config');
}

var _consumer;

function _load_consumer() {
  return _consumer = _interopRequireDefault(require('../consumer'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function getPackageDependencyValue(dependencyId, parentComponentMap, dependencyComponentMap) {
  if (!dependencyComponentMap || dependencyComponentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED) {
    return dependencyId.version;
  }
  var dependencyRootDir = dependencyComponentMap.rootDir;
  var rootDirRelative = (0, (_utils || _load_utils()).pathRelative)(parentComponentMap.rootDir, dependencyRootDir);
  return rootDirRelative.startsWith('.') ? rootDirRelative : `./${rootDirRelative}`;
}

function getRegistryPrefix() {
  return (0, (_globalConfig || _load_globalConfig()).getSync)((_constants || _load_constants()).CFG_REGISTRY_DOMAIN_PREFIX) || (_constants || _load_constants()).DEFAULT_REGISTRY_DOMAIN_PREFIX;
}

exports.addComponentsToRoot = addComponentsToRoot;
exports.changeDependenciesToRelativeSyntax = changeDependenciesToRelativeSyntax;
exports.write = write;
exports.updateAttribute = updateAttribute;
exports.addWorkspacesToPackageJson = addWorkspacesToPackageJson;