'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('./models/component'));
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('./models/version'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('./scope'));
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('./objects/repository'));
}

var _versionDependencies;

function _load_versionDependencies() {
  return _versionDependencies = _interopRequireDefault(require('./version-dependencies'));
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('./component-objects'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ComponentVersion = function () {
  function ComponentVersion(component, version) {
    _classCallCheck(this, ComponentVersion);

    this.component = component;
    this.version = version;
  }

  _createClass(ComponentVersion, [{
    key: 'getVersion',
    value: function getVersion(repository) {
      return this.component.loadVersion(this.version, repository);
    }
  }, {
    key: 'flattenedDependencies',
    value: function flattenedDependencies(repository) {
      return this.getVersion(repository).then(function (version) {
        return version.flattenedDependencies;
      });
    }
  }, {
    key: 'toId',
    value: function toId() {
      return new (_bitId || _load_bitId()).BitId({
        scope: this.component.scope,
        box: this.component.box,
        name: this.component.name,
        version: this.version
      });
    }
  }, {
    key: 'toVersionDependencies',
    value: function toVersionDependencies(scope, source, withEnvironments) {
      var _this = this;

      return this.getVersion(scope.objects).then(function (version) {
        if (!version) {
          (_logger || _load_logger()).default.debug(`toVersionDependencies, component ${_this.component.id().toString()}, version ${_this.version} not found, going to fetch from a remote`);
          if (_this.component.scope === scope.name) {
            // it should have been fetched locally, since it wasn't found, this is an error
            throw new Error(`Version ${_this.version} of ${_this.component.id().toString()} was not found in scope ${scope.name}`);
          }
          return scope.remotes().then(function (remotes) {
            var src = _this.id;
            src.scope = source;
            return scope.getExternal({ id: src, remotes, localFetch: false, withEnvironments });
          });
        }

        (_logger || _load_logger()).default.debug(`toVersionDependencies, component ${_this.component.id().toString()}, version ${_this.version} found, going to collect its dependencies`);
        return version.collectDependencies(scope, withEnvironments).then(function (dependencies) {
          return new (_versionDependencies || _load_versionDependencies()).default(_this, dependencies, source);
        });
      });
    }
  }, {
    key: 'toConsumer',
    value: function toConsumer(repo) {
      return this.component.toConsumerComponent(this.version, this.component.scope, repo);
    }
  }, {
    key: 'toObjects',
    value: function toObjects(repo) {
      var _this2 = this;

      return this.getVersion(repo).then(function (version) {
        return Promise.all([_this2.component.asRaw(repo), version.collectRaw(repo), version.asRaw(repo), repo.getScopeMetaObject()]);
      }).then(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 4),
            compObject = _ref2[0],
            objects = _ref2[1],
            version = _ref2[2],
            scopeMeta = _ref2[3];

        return new (_componentObjects || _load_componentObjects()).default(compObject, objects.concat([version, scopeMeta]));
      });
    }
  }, {
    key: 'id',
    get: function get() {
      return this.toId();
    }
  }]);

  return ComponentVersion;
}();

exports.default = ComponentVersion;